@@ -160,8 +160,19 @@
 #define SDC_CMD_GETMONGAMMACRT1		0x98980059
 #define SDC_CMD_GETMONGAMMACRT2		0x9898005a
 #define SDC_CMD_LOGQUIET		0x9898005b
+#define SDC_CMD_GETNEWGAMMABRICON	0x9898005c
+#define SDC_CMD_SETNEWGAMMABRICON	0x9898005d
+#define SDC_CMD_GETNEWGAMMABRICON2	0x9898005e
+#define SDC_CMD_SETNEWGAMMABRICON2	0x9898005f
+#define SDC_CMD_GETGETNEWGAMMACRT2	0x98980060
+#define SDC_CMD_SETGETNEWGAMMACRT2	0x98980061
+#define SDC_CMD_NEWSETVBFLAGS		0x98980062
+#define SDC_CMD_GETCRT1SATGAIN		0x98980063
+#define SDC_CMD_SETCRT1SATGAIN		0x98980064
+#define SDC_CMD_GETCRT2SATGAIN		0x98980065
+#define SDC_CMD_SETCRT2SATGAIN		0x98980066
 /* more to come, adapt MAXCOMMAND! */
-#define SDC_MAXCOMMAND			SDC_CMD_LOGQUIET
+#define SDC_MAXCOMMAND			SDC_CMD_SETCRT2SATGAIN
 
 /* in result_header */
 #define SDC_RESULT_OK  			0x66670000
@@ -285,11 +296,14 @@ SISSwitchCRT1Status(ScrnInfoPtr pScrn, int onoff, Bool quiet)
     SISPtr pSiS = SISPTR(pScrn);
     DisplayModePtr mode = pScrn->currentMode;
     ULong vbflags = pSiS->VBFlags;
+    ULong vbflags3 = pSiS->VBFlags3;
     int crt1off;
 
     /* onoff: 0=OFF, 1=ON(VGA), 2=ON(LCDA) */
 
-    /* Switching to LCDA will disable CRT2 if previously LCD or VGA */
+    /* Switching to LCDA will disable CRT2 if previously LCD or VGA,
+     * unless on a dual-vb setup
+     */
 
     /* For usability reasons, the user should not simply "lose" one
      * of his output devices in MergedFB mode. Therefore, a switch
@@ -300,6 +314,9 @@ SISSwitchCRT1Status(ScrnInfoPtr pScrn, int onoff, Bool quiet)
      * supported together with TV, not any other CRT2 type.)
      * In Non-MergedFB mode, losing one output device is not
      * considered that harmful.
+     * Update: We let the user switch off a device if currently
+     * a "clone" mode is active. Same for switching CRT1 to LCD
+     * while CRT2 is LCD or VGA.
      */
 
     /* Do NOT use this to switch from CRT1_LCDA to CRT2_LCD */
@@ -328,20 +345,24 @@ SISSwitchCRT1Status(ScrnInfoPtr pScrn, int onoff, Bool quiet)
 
 #ifdef SISMERGED
     if(pSiS->MergedFB) {
-       if(!onoff) {
-          if(!quiet) {
-             xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
-	        "CRT1 can't be switched off in MergedFB mode\n");
-	  }
-          return FALSE;
-       } else if(onoff == 2) {
-          if(vbflags & (CRT2_LCD|CRT2_VGA)) {
-	     if(!quiet) {
-	        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
-	   	   "CRT1 type can only be VGA while CRT2 is LCD or VGA\n");
+       if(((SiSMergedDisplayModePtr)mode->Private)->CRT2Position != sisClone) {
+          if(!onoff) {
+             if(!quiet) {
+                xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
+	           "CRT1 can't be switched off in MergedFB mode unless a clone mode is active\n");
 	     }
              return FALSE;
-	  }
+          } else if(onoff == 2) {
+             if(!(pSiS->SiS_SD3_Flags & SiS_SD3_SUPPORTDUALDVI)) {
+                if(vbflags & (CRT2_LCD|CRT2_VGA)) {
+	           if(!quiet) {
+	              xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
+	   	         "CRT1 type can only be VGA while CRT2 is LCD or VGA\n");
+	           }
+                   return FALSE;
+                }
+	     }
+          }
        }
        if(mode->Private) {
 	  mode = ((SiSMergedDisplayModePtr)mode->Private)->CRT1;
@@ -350,13 +371,19 @@ SISSwitchCRT1Status(ScrnInfoPtr pScrn, int onoff, Bool quiet)
 #endif
 
     vbflags &= ~(DISPTYPE_CRT1 | SINGLE_MODE | MIRROR_MODE | CRT1_LCDA);
+    vbflags3 &= ~(VB3_CRT1_TV | VB3_CRT1_LCD | VB3_CRT1_VGA);
     crt1off = 1;
     if(onoff > 0) {
        vbflags |= DISPTYPE_CRT1;
        crt1off = 0;
        if(onoff == 2) {
 	  vbflags |= CRT1_LCDA;
-	  vbflags &= ~(CRT2_LCD|CRT2_VGA);
+	  vbflags3 |= VB3_CRT1_LCD;
+	  if(!(pSiS->SiS_SD3_Flags & SiS_SD3_SUPPORTDUALDVI)) {
+	     vbflags &= ~(CRT2_LCD|CRT2_VGA);
+	  }
+       } else {
+          vbflags3 |= VB3_CRT1_VGA;
        }
        /* Remember: Dualhead not supported */
        if(vbflags & CRT2_ENABLE) vbflags |= MIRROR_MODE;
@@ -377,6 +404,7 @@ SISSwitchCRT1Status(ScrnInfoPtr pScrn, int onoff, Bool quiet)
 
     pSiS->CRT1off = crt1off;
     pSiS->VBFlags = pSiS->VBFlags_backup = vbflags;
+    pSiS->VBFlags3 = pSiS->VBFlags_backup3 = vbflags3;
 
     /* Sync the accelerators */
     (*pSiS->SyncAccel)(pScrn);
@@ -444,6 +472,8 @@ SISSwitchCRT2Type(ScrnInfoPtr pScrn, ULong newvbflags, Bool quiet)
      * supported together with TV, not any other CRT2 type.)
      * In Non-MergedFB mode, losing one output device is not
      * considered that harmful.
+     * Update: We allow this if currently a "clone" display mode
+     * is active.
      */
 
     /* Only on 300 and 315/330/340 series */
@@ -476,19 +506,24 @@ SISSwitchCRT2Type(ScrnInfoPtr pScrn, ULong newvbflags, Bool quiet)
 
 #ifdef SISMERGED
     if(pSiS->MergedFB) {
-       if(!(newvbflags & CRT2_ENABLE)) {
-          if(!quiet) {
-	     xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
-	        "CRT2 can't be switched off in MergedFB mode\n");
-	  }
-	  return FALSE;
-       }
-       if((newvbflags & (CRT2_LCD|CRT2_VGA)) && (newvbflags & CRT1_LCDA)) {
-          if(!quiet) {
-	     xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
-	        "CRT2 type can only be TV while in LCD-via-CRT1 mode\n");
-	  }
-	  return FALSE;
+       if((mode->Private) &&
+          ((SiSMergedDisplayModePtr)mode->Private)->CRT2Position != sisClone) {
+          if(!(newvbflags & CRT2_ENABLE)) {
+             if(!quiet) {
+	        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
+	           "CRT2 can't be switched off in MergedFB mode unless a clone mode is active\n");
+	     }
+	     return FALSE;
+          }
+          if(!(pSiS->SiS_SD3_Flags & SiS_SD3_SUPPORTDUALDVI)) {
+             if((newvbflags & (CRT2_LCD|CRT2_VGA)) && (newvbflags & CRT1_LCDA)) {
+                if(!quiet) {
+	           xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
+	              "CRT2 type can only be TV while in LCD-via-CRT1 mode\n");
+	        }
+	        return FALSE;
+	     }
+          }
        }
        hcm = pSiS->HaveCustomModes2;
        if(mode->Private) {
@@ -506,8 +541,10 @@ SISSwitchCRT2Type(ScrnInfoPtr pScrn, ULong newvbflags, Bool quiet)
     }
 
     /* CRT2_LCD and CRT2_VGA overrule LCDA (in non-MergedFB mode) */
-    if(newvbflags & (CRT2_LCD|CRT2_VGA)) {
-       newvbflags &= ~CRT1_LCDA;
+    if(!(pSiS->SiS_SD3_Flags & SiS_SD3_SUPPORTDUALDVI)) {
+       if(newvbflags & (CRT2_LCD|CRT2_VGA)) {
+          newvbflags &= ~CRT1_LCDA;
+       }
     }
 
     /* Check if the current mode is suitable for desired output device (if any) */
@@ -542,6 +579,15 @@ SISSwitchCRT2Type(ScrnInfoPtr pScrn, ULong newvbflags, Bool quiet)
     return TRUE;
 }
 
+static Bool
+SISSwitchOutputType(ScrnInfoPtr pScrn, ULong newvbflags, ULong newvbflags3,
+		ULong newvbflags4, Bool quiet)
+{
+    /* For future use */
+
+    return FALSE;
+}
+
 static ULong
 SiSCondToVBFlags(UShort cond, ULong GivenVBFlags)
 {
@@ -566,6 +612,10 @@ SiSCondToVBFlags(UShort cond, ULong GivenVBFlags)
 		vbflags |= (TV_YPBPR | TV_YPBPR525I);
 	  else if((cond & SiS_CF2_TVSPECMASK) == SiS_CF2_TVYPBPR525P)
 		vbflags |= (TV_YPBPR | TV_YPBPR525P);
+	  else if((cond & SiS_CF2_TVSPECMASK) == SiS_CF2_TVYPBPR625I)
+		vbflags |= (TV_YPBPR | TV_YPBPR625I);
+	  else if((cond & SiS_CF2_TVSPECMASK) == SiS_CF2_TVYPBPR625P)
+		vbflags |= (TV_YPBPR | TV_YPBPR625P);
 	  else if((cond & SiS_CF2_TVSPECMASK) == SiS_CF2_TVYPBPR750P)
 		vbflags |= (TV_YPBPR | TV_YPBPR750P);
 	  else if((cond & SiS_CF2_TVSPECMASK) == SiS_CF2_TVYPBPR1080I)
@@ -911,6 +961,10 @@ SiSHandleSiSDirectCommand(xSiSCtrlCommandReply *sdcbuf)
       sdcbuf->sdc_result[3] = pSiS->SiS_SD2_Flags;
       sdcbuf->sdc_result[4] = pSiS->detectedCRT2Devices;
       sdcbuf->sdc_result[5] = pSiS->VBFlags2;
+      sdcbuf->sdc_result[6] = pSiS->SiS_SD3_Flags;
+      sdcbuf->sdc_result[7] = pSiS->SiS_SD4_Flags;
+      sdcbuf->sdc_result[8] = pSiS->VBFlags3;
+      sdcbuf->sdc_result[9] = pSiS->VBFlags4;
       break;
 
    case SDC_CMD_GETVBFLAGSVERSION:
@@ -920,6 +974,8 @@ SiSHandleSiSDirectCommand(xSiSCtrlCommandReply *sdcbuf)
    case SDC_CMD_GETVBFLAGS:
       sdcbuf->sdc_result[0] = pSiS->VBFlags;
       sdcbuf->sdc_result[1] = pSiS->VBFlags2;
+      sdcbuf->sdc_result[2] = pSiS->VBFlags3;
+      sdcbuf->sdc_result[3] = pSiS->VBFlags4;
       break;
 
    case SDC_CMD_CHECKMODEFORCRT2:
@@ -957,6 +1013,20 @@ SiSHandleSiSDirectCommand(xSiSCtrlCommandReply *sdcbuf)
 #endif
       break;
 
+   case SDC_CMD_NEWSETVBFLAGS:
+#ifdef SISDUALHEAD
+      if(!pSiS->DualHeadMode) {
+#endif
+	 if(pSiS->xv_sisdirectunlocked) {
+	    SISSwitchOutputType(pScrn, (ULong)sdcbuf->sdc_parm[0], (ULong)sdcbuf->sdc_parm[1],
+	    		(ULong)sdcbuf->sdc_parm[2], pSiS->SCLogQuiet);
+	    if(pPriv) SISUpdateVideoParms(pSiS, pPriv);
+	 } else sdcbuf->sdc_result_header = SDC_RESULT_NOPERM;
+#ifdef SISDUALHEAD
+      } else sdcbuf->sdc_result_header = SDC_RESULT_NOPERM;
+#endif
+      break;
+
    case SDC_CMD_GETDETECTEDDEVICES:
       sdcbuf->sdc_result[0] = pSiS->detectedCRT2Devices;
       break;
@@ -1260,6 +1330,15 @@ SiSHandleSiSDirectCommand(xSiSCtrlCommandReply *sdcbuf)
       sdcbuf->sdc_result[2] = pSiS->GammaBriB;
       break;
 
+   case SDC_CMD_GETNEWGAMMABRICON:  /* no xv pendant */
+      sdcbuf->sdc_result[0] = (CARD32)(((int)(pSiS->NewGammaBriR * 1000.0)) + 1000);
+      sdcbuf->sdc_result[1] = (CARD32)(((int)(pSiS->NewGammaBriG * 1000.0)) + 1000);
+      sdcbuf->sdc_result[2] = (CARD32)(((int)(pSiS->NewGammaBriB * 1000.0)) + 1000);
+      sdcbuf->sdc_result[3] = (CARD32)(((int)(pSiS->NewGammaConR * 1000.0)) + 1000);
+      sdcbuf->sdc_result[4] = (CARD32)(((int)(pSiS->NewGammaConG * 1000.0)) + 1000);
+      sdcbuf->sdc_result[5] = (CARD32)(((int)(pSiS->NewGammaConB * 1000.0)) + 1000);
+      break;
+
    case SDC_CMD_SETGAMMABRIGHTNESS:  /* xv_BRx, xv_PBx */
       if(sdcbuf->sdc_parm[0] < 100 || sdcbuf->sdc_parm[0] > 10000 ||
 	 sdcbuf->sdc_parm[1] < 100 || sdcbuf->sdc_parm[1] > 10000 ||
@@ -1269,6 +1348,26 @@ SiSHandleSiSDirectCommand(xSiSCtrlCommandReply *sdcbuf)
 	 pSiS->GammaBriR = sdcbuf->sdc_parm[0];
 	 pSiS->GammaBriG = sdcbuf->sdc_parm[1];
 	 pSiS->GammaBriB = sdcbuf->sdc_parm[2];
+	 pSiS->NewGammaBriR = pSiS->NewGammaBriG = pSiS->NewGammaBriB = 0.0;
+	 pSiS->NewGammaConR = pSiS->NewGammaConG = pSiS->NewGammaConB = 0.0;
+	 pSiS->SiS_SD3_Flags |= SiS_SD3_OLDGAMMAINUSE;
+      } else sdcbuf->sdc_result_header = SDC_RESULT_NOPERM;
+      break;
+
+   case SDC_CMD_SETNEWGAMMABRICON:  /* no xv pendant */
+      if(sdcbuf->sdc_parm[0] > 2000 || sdcbuf->sdc_parm[1] > 2000 ||
+	 sdcbuf->sdc_parm[2] > 2000 || sdcbuf->sdc_parm[3] > 2000 ||
+	 sdcbuf->sdc_parm[4] > 2000 || sdcbuf->sdc_parm[5] > 2000) {
+	 sdcbuf->sdc_result_header = SDC_RESULT_INVAL;
+      } else if(pSiS->xv_sisdirectunlocked) {
+	 pSiS->NewGammaBriR = ((float)((int)sdcbuf->sdc_parm[0] - 1000)) / 1000.0;
+	 pSiS->NewGammaBriG = ((float)((int)sdcbuf->sdc_parm[1] - 1000)) / 1000.0;
+	 pSiS->NewGammaBriB = ((float)((int)sdcbuf->sdc_parm[2] - 1000)) / 1000.0;
+	 pSiS->NewGammaConR = ((float)((int)sdcbuf->sdc_parm[3] - 1000)) / 1000.0;
+	 pSiS->NewGammaConG = ((float)((int)sdcbuf->sdc_parm[4] - 1000)) / 1000.0;
+	 pSiS->NewGammaConB = ((float)((int)sdcbuf->sdc_parm[5] - 1000)) / 1000.0;
+	 pSiS->GammaBriR = pSiS->GammaBriG = pSiS->GammaBriB = 1000;
+	 pSiS->SiS_SD3_Flags &= ~SiS_SD3_OLDGAMMAINUSE;
       } else sdcbuf->sdc_result_header = SDC_RESULT_NOPERM;
       break;
 
@@ -1286,17 +1385,61 @@ SiSHandleSiSDirectCommand(xSiSCtrlCommandReply *sdcbuf)
       sdcbuf->sdc_result[2] = pSiS->GammaBriB;
       break;
 
+   case SDC_CMD_GETNEWGAMMABRICON2: /* no xv pendant */
+#ifdef SISDUALHEAD
+      if(pSiS->DualHeadMode) {
+         sdcbuf->sdc_result[0] = (CARD32)(((int)(pSiSEnt->NewGammaBriR * 1000.0)) + 1000);
+	 sdcbuf->sdc_result[1] = (CARD32)(((int)(pSiSEnt->NewGammaBriG * 1000.0)) + 1000);
+	 sdcbuf->sdc_result[2] = (CARD32)(((int)(pSiSEnt->NewGammaBriB * 1000.0)) + 1000);
+	 sdcbuf->sdc_result[3] = (CARD32)(((int)(pSiSEnt->NewGammaConR * 1000.0)) + 1000);
+	 sdcbuf->sdc_result[4] = (CARD32)(((int)(pSiSEnt->NewGammaConG * 1000.0)) + 1000);
+	 sdcbuf->sdc_result[5] = (CARD32)(((int)(pSiSEnt->NewGammaConB * 1000.0)) + 1000);
+	 break;
+      }
+#endif
+      sdcbuf->sdc_result[0] = (CARD32)(((int)(pSiS->NewGammaBriR * 1000.0)) + 1000);
+      sdcbuf->sdc_result[1] = (CARD32)(((int)(pSiS->NewGammaBriG * 1000.0)) + 1000);
+      sdcbuf->sdc_result[2] = (CARD32)(((int)(pSiS->NewGammaBriB * 1000.0)) + 1000);
+      sdcbuf->sdc_result[3] = (CARD32)(((int)(pSiS->NewGammaConR * 1000.0)) + 1000);
+      sdcbuf->sdc_result[4] = (CARD32)(((int)(pSiS->NewGammaConG * 1000.0)) + 1000);
+      sdcbuf->sdc_result[5] = (CARD32)(((int)(pSiS->NewGammaConB * 1000.0)) + 1000);
+      break;
+
    case SDC_CMD_SETGAMMABRIGHTNESS2: /* xv_BRx2, xv_PBx2 */
       if(sdcbuf->sdc_parm[0] < 100 || sdcbuf->sdc_parm[0] > 10000 ||
 	 sdcbuf->sdc_parm[1] < 100 || sdcbuf->sdc_parm[1] > 10000 ||
 	 sdcbuf->sdc_parm[2] < 100 || sdcbuf->sdc_parm[2] > 10000) {
 	 sdcbuf->sdc_result_header = SDC_RESULT_INVAL;
       } else if(pSiS->xv_sisdirectunlocked) {
+         pSiS->SiS_SD3_Flags |= SiS_SD3_OLDGAMMAINUSE;
 #ifdef SISDUALHEAD
 	 if(pSiS->DualHeadMode) {
 	    pSiSEnt->GammaBriR = sdcbuf->sdc_parm[0];
 	    pSiSEnt->GammaBriG = sdcbuf->sdc_parm[1];
 	    pSiSEnt->GammaBriB = sdcbuf->sdc_parm[2];
+	    pSiSEnt->NewGammaBriR = pSiSEnt->NewGammaBriG = pSiSEnt->NewGammaBriB = 0.0;
+	    pSiSEnt->NewGammaConR = pSiSEnt->NewGammaConG = pSiSEnt->NewGammaConB = 0.0;
+	 }
+#endif
+      } else sdcbuf->sdc_result_header = SDC_RESULT_NOPERM;
+      break;
+
+   case SDC_CMD_SETNEWGAMMABRICON2: /* no xv pendant */
+      if(sdcbuf->sdc_parm[0] > 2000 || sdcbuf->sdc_parm[1] > 2000 ||
+	 sdcbuf->sdc_parm[2] > 2000 || sdcbuf->sdc_parm[3] > 2000 ||
+	 sdcbuf->sdc_parm[4] > 2000 || sdcbuf->sdc_parm[5] > 2000) {
+	 sdcbuf->sdc_result_header = SDC_RESULT_INVAL;
+      } else if(pSiS->xv_sisdirectunlocked) {
+         pSiS->SiS_SD3_Flags &= ~SiS_SD3_OLDGAMMAINUSE;
+#ifdef SISDUALHEAD
+	 if(pSiS->DualHeadMode) {
+	    pSiSEnt->NewGammaBriR = ((float)((int)sdcbuf->sdc_parm[0] - 1000)) / 1000.0;
+	    pSiSEnt->NewGammaBriG = ((float)((int)sdcbuf->sdc_parm[1] - 1000)) / 1000.0;
+	    pSiSEnt->NewGammaBriB = ((float)((int)sdcbuf->sdc_parm[2] - 1000)) / 1000.0;
+	    pSiSEnt->NewGammaConR = ((float)((int)sdcbuf->sdc_parm[3] - 1000)) / 1000.0;
+	    pSiSEnt->NewGammaConG = ((float)((int)sdcbuf->sdc_parm[4] - 1000)) / 1000.0;
+	    pSiSEnt->NewGammaConB = ((float)((int)sdcbuf->sdc_parm[5] - 1000)) / 1000.0;
+	    pSiSEnt->GammaBriR = pSiSEnt->GammaBriG = pSiSEnt->GammaBriB = 1000;
 	 }
 #endif
       } else sdcbuf->sdc_result_header = SDC_RESULT_NOPERM;
@@ -1311,6 +1454,18 @@ SiSHandleSiSDirectCommand(xSiSCtrlCommandReply *sdcbuf)
       sdcbuf->sdc_result[5] = pSiS->GammaBriB2;
       break;
 
+   case SDC_CMD_GETGETNEWGAMMACRT2:
+      sdcbuf->sdc_result[0] = (ULong)(pSiS->GammaR2 * 1000);
+      sdcbuf->sdc_result[1] = (ULong)(pSiS->GammaG2 * 1000);
+      sdcbuf->sdc_result[2] = (ULong)(pSiS->GammaB2 * 1000);
+      sdcbuf->sdc_result[3] = (CARD32)(((int)(pSiS->NewGammaBriR2 * 1000.0)) + 1000);
+      sdcbuf->sdc_result[4] = (CARD32)(((int)(pSiS->NewGammaBriG2 * 1000.0)) + 1000);
+      sdcbuf->sdc_result[5] = (CARD32)(((int)(pSiS->NewGammaBriB2 * 1000.0)) + 1000);
+      sdcbuf->sdc_result[6] = (CARD32)(((int)(pSiS->NewGammaConR2 * 1000.0)) + 1000);
+      sdcbuf->sdc_result[7] = (CARD32)(((int)(pSiS->NewGammaConG2 * 1000.0)) + 1000);
+      sdcbuf->sdc_result[8] = (CARD32)(((int)(pSiS->NewGammaConB2 * 1000.0)) + 1000);
+      break;
+
    case SDC_CMD_SETGETGAMMACRT2:
       if(sdcbuf->sdc_parm[0] < 100 || sdcbuf->sdc_parm[0] > 10000 ||
 	 sdcbuf->sdc_parm[1] < 100 || sdcbuf->sdc_parm[1] > 10000 ||
@@ -1326,6 +1481,33 @@ SiSHandleSiSDirectCommand(xSiSCtrlCommandReply *sdcbuf)
 	 pSiS->GammaBriR2 = sdcbuf->sdc_parm[3];
 	 pSiS->GammaBriG2 = sdcbuf->sdc_parm[4];
 	 pSiS->GammaBriB2 = sdcbuf->sdc_parm[5];
+	 pSiS->NewGammaBriR2 = pSiS->NewGammaBriG2 = pSiS->NewGammaBriB2 = 0.0;
+	 pSiS->NewGammaConR2 = pSiS->NewGammaConG2 = pSiS->NewGammaConB2 = 0.0;
+	 pSiS->SiS_SD3_Flags |= SiS_SD3_OLDGAMMAINUSE;
+	 SiS_UpdateGammaCRT2(pScrn);
+      } else sdcbuf->sdc_result_header = SDC_RESULT_NOPERM;
+      break;
+
+   case SDC_CMD_SETGETNEWGAMMACRT2:
+      if(sdcbuf->sdc_parm[0] < 100 || sdcbuf->sdc_parm[0] > 10000 ||
+	 sdcbuf->sdc_parm[1] < 100 || sdcbuf->sdc_parm[1] > 10000 ||
+	 sdcbuf->sdc_parm[2] < 100 || sdcbuf->sdc_parm[2] > 10000 ||
+	 sdcbuf->sdc_parm[3] > 2000 || sdcbuf->sdc_parm[4] > 2000 ||
+	 sdcbuf->sdc_parm[5] > 2000 || sdcbuf->sdc_parm[6] > 2000 ||
+	 sdcbuf->sdc_parm[7] > 2000 || sdcbuf->sdc_parm[8] > 2000) {
+	 sdcbuf->sdc_result_header = SDC_RESULT_INVAL;
+      } else if(pSiS->xv_sisdirectunlocked) {
+         pSiS->GammaR2 = (float)sdcbuf->sdc_parm[0] / (float)1000;
+	 pSiS->GammaG2 = (float)sdcbuf->sdc_parm[1] / (float)1000;
+	 pSiS->GammaB2 = (float)sdcbuf->sdc_parm[2] / (float)1000;
+	 pSiS->NewGammaBriR2 = ((float)((int)sdcbuf->sdc_parm[3] - 1000)) / 1000.0;
+	 pSiS->NewGammaBriG2 = ((float)((int)sdcbuf->sdc_parm[4] - 1000)) / 1000.0;
+	 pSiS->NewGammaBriB2 = ((float)((int)sdcbuf->sdc_parm[5] - 1000)) / 1000.0;
+	 pSiS->NewGammaConR2 = ((float)((int)sdcbuf->sdc_parm[6] - 1000)) / 1000.0;
+	 pSiS->NewGammaConG2 = ((float)((int)sdcbuf->sdc_parm[7] - 1000)) / 1000.0;
+	 pSiS->NewGammaConB2 = ((float)((int)sdcbuf->sdc_parm[8] - 1000)) / 1000.0;
+	 pSiS->GammaBriR2 = pSiS->GammaBriG2 = pSiS->GammaBriB2 = 1000;
+	 pSiS->SiS_SD3_Flags &= ~SiS_SD3_OLDGAMMAINUSE;
 	 SiS_UpdateGammaCRT2(pScrn);
       } else sdcbuf->sdc_result_header = SDC_RESULT_NOPERM;
       break;
@@ -1688,6 +1870,16 @@ SiSHandleSiSDirectCommand(xSiSCtrlCommandReply *sdcbuf)
       pSiS->SCLogQuiet = sdcbuf->sdc_parm[0] ? TRUE : FALSE;
       break;
 
+   case SDC_CMD_GETCRT1SATGAIN:
+      sdcbuf->sdc_result[0] = SiS_GetSISCRT1SaturationGain(pScrn);
+      break;
+
+   case SDC_CMD_SETCRT1SATGAIN:
+      if(pSiS->xv_sisdirectunlocked) {
+	 SiS_SetSISCRT1SaturationGain(pScrn, (int)sdcbuf->sdc_parm[0]);
+      } else sdcbuf->sdc_result_header = SDC_RESULT_NOPERM;
+      break;
+
    default:
       sdcbuf->sdc_result_header = SDC_RESULT_UNDEFCMD;
    }
